function [ Ip ] = triode(Vgk, Vpk, mu, k_x, k_g1, k_p, k_vb)
%TRIODE Model of a triode vacuum-tube.
%   Ip = TRIODE(Vgk, Vpk, mu, k_x, k_g1, k_p, k_vb) returns the plate current
%   Ip of a triode vacuum-tube as a nonlinear function of the grid-to-
%   cathode voltage Vgk and the plate-to-cathode voltage Vpk. The imple-
%   mented tube model was developed by Norman Koren [1] and is parametrized
%   by the amplification factor mu and the constants k_x, k_g1, k_p and
%   k_vb. Parameters for some popular tubes may be retrieved by a call
%   of GET_TRIODE_PARAM.
%
%   This model only accounts for the nonlinear relationship whereas
%   the parasitic capacitances and the diode between grid and cathode
%   are not considered.
%
%   [1] http://www.normankoren.com/Audio/Tubemodspice_article.html

% Revision:
%   2011/06/02 - Initial version, Matthias Hotz
%
% Copyright 2011 by Matthias Hotz
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.

  V1 = (Vpk/k_p)*log(1 + exp(k_p*(1/mu + Vgk/sqrt(k_vb + Vpk^2))));
  Ip = V1^k_x/k_g1*(1 + sign(V1));

end
