%--------------------------------------------------------------------------
% Simulation of the wave digital tube stage model
%
% Revision:
%   2011/06/02 - Initial version, Matthias Hotz
%   2011/06/07 - Added file management, upsampling and execution of the
%                simulation, Matthias Hotz
%   2011/07/11 - Added export of grid-to-cathode voltage, Matthias Hotz
%
% Copyright 2011 by Matthias Hotz
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Simulation parameters

% Input and output wave files
wave_in = '..\audio\input\sine_1khz.wav';
wave_vout = '..\audio\output\vout_wd.wav';
wave_vgk = '..\audio\output\vgk_wd.wav';

% Input wave file pre-gain before it is fed into the circuit
pregain = 3;

% Maximum simulation stop time
T_max = 17;

% Upsampling factor
k_upsample = 1;

% Resolution of output wave file in bits
res_out = 24;

%--------------------------------------------------------------------------
% Tube stage circuit parameters

% Grid circuit
Rin = 1;
Ci = 100e-9;
Rci = 10e3;    % Substitute for Ci used for audio files, see Simulink model
Ri = 1e6;
Rg = 20e3;

% Cathode circuit
Rk = 1e3;
Ck = 10e-6;

% Plate circuit
Rp = 100e3;
Vs = 250;
Co = 10e-9;
Ro = 1e6;

% Tube: Grid-to-cathode diode approximation
Rgkc = 2.7e3;
Rgkb = 100e9;

% Tube: Characteristic values
[mu, k_x, k_g1, k_p, k_vb] = get_triode_param('12AX7');

%--------------------------------------------------------------------------
% Preparation for simulation

% Read input wave file
[vin, fs, res] = wavread(wave_in);

% Original sampling time
T0 = 1/fs;

% Upsample signal
if k_upsample > 1
  vin = interp(vin, k_upsample);
  T = T0/k_upsample;
else
  T = T0;
end

% Generate 2D-matrix with timestamp for use in Simulink
t_vin = 0:T:(length(vin)-1)*T;
vin = [t_vin(:), vin(:)];

% Determine desired simulation duration
if T_max <= 0 || T_max > max(t_vin)
  T_max = max(t_vin);
end

%--------------------------------------------------------------------------
% Run simulation

disp('Simulating wave digital tube stage circuit:');
disp(['  Input file:        ' wave_in]);
disp(['  Output file:       ' wave_vout]);
fprintf('  Pre-gain:          %.3f\n', pregain);
fprintf('  Duration:          %.3fs\n', T_max);
fprintf('  Upsampling factor: %d\n\n', k_upsample);

% Run wave digital model in Simulink
tic;
sim('tube_stage_model', [0 T_max]);
t_sim = toc;

fprintf('Simulated circuit in %.2fs.\n\n', t_sim);

%--------------------------------------------------------------------------
% Store output signal

% Remove the very first sample because Vgk which controls Rpk is delayed by
% one sample, thus also the output signal
vout = vout(2:end);

% Downsample signal
if k_upsample > 1
  vout = decimate(vout, k_upsample);
end

% Scale output signal and grid-to-cathode voltage and write to file
wavwrite(vout/Vs, fs, res_out, wave_vout);
wavwrite(vgk/Vs, fs, res_out, wave_vgk);
